/*
 * (c) 2025 Jens Mueller
 *
 * JBASICODE
 *
 * Stack item of FOR instruction
 */

package jbasicode.bc;

import java.util.List;


public class ForStackItem extends BCStackItem
{
  private String       varName;
  private List<Number> varIndices;
  private Number       endValue;
  private Number       stepSize;
  private BCStackItem  item;
  private String       text;


  public ForStackItem(
		String       varName,
		List<Number> varIndices,
		Number       endValue,
		Number       stepSize,
		BCStackItem  item )
  {
    super(
	item.getLineIndex(),
	item.getBCLineNumber(),
	item.getCharIndex() );
    this.varName    = varName;
    this.varIndices = varIndices;
    this.endValue   = endValue;
    this.stepSize   = stepSize;

    StringBuilder buf = new StringBuilder();
    buf.append( "FOR " );
    buf.append( varName );
    if( varIndices != null ) {
      int n = varIndices.size();
      if( n > 0 ) {
	buf.append( '(' );
	for( int i = 0; i < n; i++ ) {
	  if( i > 0 ) {
	    buf.append( ',' );
	  }
	  buf.append( varIndices.get( i ) );
	}
	buf.append( ')' );
      }
    }
    if( endValue != null ) {
      buf.append( " TO " );
      buf.append( endValue );
      if( stepSize != null ) {
	if( !(stepSize instanceof Integer) || (stepSize.intValue() != 1) ) {
	  buf.append( " STEP " );
	  buf.append( stepSize );
	}
      }
    }
    buf.append( '\u0020' );
    buf.append( super.toString() );
    this.text = buf.toString();
  }


  public Number getEndValue()
  {
    return this.endValue;
  }


  public Number getStepSize()
  {
    return this.stepSize;
  }


  public List<Number> getVarIndices()
  {
    return this.varIndices;
  }


  public String getVarName()
  {
    return this.varName;
  }


  @Override
  public String toString()
  {
    return this.text;
  }
}
