/*
 * (c) 2025 Jens Mueller
 *
 * JBASICODE
 *
 * Screen Component
 */

package jbasicode.ui;

import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JComponent;
import jbasicode.bc.BCScreen;


public class ScreenFld
		extends JComponent
		implements KeyListener, MouseListener
{
  private boolean  notified;
  private BCScreen bcScreen;


  public ScreenFld( BCScreen bcScreen )
  {
    this.bcScreen = bcScreen;
    this.notified = false;
  }


  public void fireSetDisabled()
  {
    EventQueue.invokeLater( ()->setEnabled( false ) );
  }


  public void updPrefSize()
  {
    int w = this.bcScreen.getWidth();
    int h = this.bcScreen.getHeight();
    Insets insets = getInsets();
    if( insets != null ) {
      w = w + insets.left + insets.right;
      h = h + insets.top + insets.bottom;
    }
    setPreferredSize( new Dimension( w + 1, h + 1 ) );
  }


  @Override
  public void addNotify()
  {
    super.addNotify();
    if( !this.notified ) {
      this.notified = true;
      addKeyListener( this );
      addMouseListener( this );
    }
  }


  @Override
  public boolean isFocusable()
  {
    return true;
  }


  @Override
  public void paintComponent( Graphics graphics )
  {
    if( isEnabled() ) {
      int x = 1;
      int y = 1;
      int w = getWidth() - 1;
      int h = getHeight() - 1;
      Insets insets = getInsets();
      if( insets != null ) {
	x += insets.left;
	y += insets.top;
	w -= insets.left;
	w -= insets.right;
	h -= insets.top;
	h -= insets.bottom;
      }
      if( (w > 0) && (h > 0) ) {
	graphics.translate( insets.left, insets.top );
	this.bcScreen.paint( graphics, w, h );
      }
    }
  }


  @Override
  public void removeNotify()
  {
    if( this.notified ) {
      this.notified = false;
      removeKeyListener( this );
      removeMouseListener( this );
    }
    super.removeNotify();
  }


	/* --- KeyListener --- */

  @Override
  public void keyPressed( KeyEvent e )
  {
    int keyCode = 0;
    switch( e.getKeyCode() ) {
      case KeyEvent.VK_F1:
	keyCode = -1;
	break;
      case KeyEvent.VK_F2:
	keyCode = -2;
	break;
      case KeyEvent.VK_ENTER:
	keyCode = 13;
	break;
      case KeyEvent.VK_ESCAPE:
	keyCode = 27;
	break;
      case KeyEvent.VK_LEFT:
	keyCode = 28;
	break;
      case KeyEvent.VK_RIGHT:
	keyCode = 29;
	break;
      case KeyEvent.VK_DOWN:
	keyCode = 30;
	break;
      case KeyEvent.VK_UP:
	keyCode = 31;
	break;
      case KeyEvent.VK_BACK_SPACE:
      case KeyEvent.VK_DELETE:
	keyCode = 127;
	break;
    }
    if( keyCode != 0 ) {
      this.bcScreen.setCurKeyChar( keyCode );
    }
  }


  @Override
  public void keyReleased( KeyEvent e )
  {
    this.bcScreen.setCurKeyChar( 0 );
    e.consume();
  }


  @Override
  public void keyTyped( KeyEvent e )
  {
    this.bcScreen.keyTyped( e.getKeyChar() );
    e.consume();
  }


	/* --- MouseListener --- */

  @Override
  public void mouseClicked( MouseEvent e )
  {
    // empty
  }


  @Override
  public void mouseEntered( MouseEvent e )
  {
    // empty
  }


  @Override
  public void mouseExited( MouseEvent e )
  {
    // empty
  }


  @Override
  public void mousePressed( MouseEvent e )
  {
    if( e.getComponent() == this )
      requestFocus();
  }


  @Override
  public void mouseReleased( MouseEvent e )
  {
    // empty
  }
}
