/*
 * (c) 2025 Jens Mueller
 *
 * JBASICODE
 *
 * Dialog to edit the application setiings
 */

package jbasicode.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import jbasicode.Main;
import jbasicode.bc.BCRuntime;
import jbasicode.bc.BCScreen;
import jbasicode.bc.ProgramParser;


public class SettingsDlg extends JDialog implements ActionListener
{
  private MainFrm           mainFrm;
  private boolean           notified;
  private JTabbedPane       tabbedPane;
  private JCheckBox         cbBCInputReverseCase;
  private JCheckBox         cbBCParserIgnoreCase;
  private JCheckBox         cbWaitDurationWhenSoundOff;
  private JCheckBox         cbCursorAlwaysVisible;
  private JCheckBox         cbGoto950withoutCLS;
  private JCheckBox         cbWarnNonAscii;
  private JCheckBox         cbWarnSpecViolated;
  private JCheckBox         cbWarnRedeclaration;
  private JCheckBox         cbWarnVarNotInitialized;
  private JComboBox<String> comboAspectRatio;
  private JComboBox<String> comboColorAtStartBg;
  private JComboBox<String> comboColorAtStartFg;
  private JRadioButton      rbEorCR;
  private JRadioButton      rbEorCRNL;
  private JRadioButton      rbEorNL;
  private JButton           btnOK;
  private JButton           btnReset;
  private JButton           btnCancel;


  public static void open( MainFrm mainFrm )
  {
    (new SettingsDlg( mainFrm )).setVisible( true );
  }


  @Override
  public void addNotify()
  {
    super.addNotify();
    if( !this.notified ) {
      this.notified = true;
      this.btnOK.addActionListener( this );
      this.btnReset.addActionListener( this );
      this.btnCancel.addActionListener( this );
    }
  }


  @Override
  public void removeNotify()
  {
    if( this.notified ) {
      this.notified = false;
      this.btnOK.removeActionListener( this );
      this.btnReset.removeActionListener( this );
      this.btnCancel.removeActionListener( this );
    }
    super.removeNotify();
  }


	/* --- ActionListener --- */

  @Override
  public void actionPerformed( ActionEvent e )
  {
    Object src = e.getSource();
    if( src == this.btnOK ) {
      doApply();
    } else if( src == this.btnReset ) {
      doReset();
    }
    else if( src == this.btnCancel ) {
      doClose();
    }
  }


	/* --- Constructor --- */

  private SettingsDlg( MainFrm mainFrm )
  {
    super( mainFrm );
    this.mainFrm  = mainFrm;
    this.notified = false;
    setDefaultCloseOperation( JDialog.DISPOSE_ON_CLOSE );
    setTitle( Main.getText( "title.settings" ) );


    // Window content
    setLayout( new GridBagLayout() );

    GridBagConstraints gbc = new GridBagConstraints(
					0, 0,
					1, 1,
					1.0, 1.0,
					GridBagConstraints.CENTER,
					GridBagConstraints.BOTH,
					new Insets( 5, 5, 5, 5 ),
					0, 0 );

    this.tabbedPane = new JTabbedPane( JTabbedPane.TOP );
    add( this.tabbedPane, gbc );


    // Tab general settings
    JPanel panelGeneral = new JPanel( new GridBagLayout() );
    this.tabbedPane.addTab(
		Main.getText( "title.settings.general" ),
		panelGeneral );

    GridBagConstraints gbcGeneral = new GridBagConstraints(
					0, 0,
					1, 1,
					0.0, 0.0,
					GridBagConstraints.WEST,
					GridBagConstraints.NONE,
					new Insets( 5, 5, 0, 5 ),
					0, 0 );

    this.cbBCInputReverseCase = new JCheckBox(
		Main.getText( "option.bc.input.reverse_case" ),
		Main.getBooleanProperty(
			BCScreen.PROP_INPUT_REVERSE_CASE,
			BCScreen.DEFAULT_INPUT_REVERSE_CASE ) );
    gbcGeneral.gridy++;
    panelGeneral.add( this.cbBCInputReverseCase, gbcGeneral );

    this.cbBCParserIgnoreCase = new JCheckBox(
		Main.getText( "option.bc.parse.ignore_case" ),
		Main.getBooleanProperty(
			ProgramParser.PROP_IGNORE_CASE,
			ProgramParser.DEFAULT_IGNORE_CASE ) );
    gbcGeneral.insets.top = 0;
    gbcGeneral.gridy++;
    panelGeneral.add( this.cbBCParserIgnoreCase, gbcGeneral );

    this.cbWaitDurationWhenSoundOff = new JCheckBox(
		Main.getText( "option.wait_duration_when_sound_off" ),
		BCRuntime.getWaitDurationWhenSoundOffSetting() );
    gbcGeneral.insets.top    = 0;
    gbcGeneral.insets.bottom = 5;
    gbcGeneral.gridy++;
    panelGeneral.add( this.cbWaitDurationWhenSoundOff, gbcGeneral );


    // Tab screen settings
    JPanel panelScreen = new JPanel( new GridBagLayout() );
    this.tabbedPane.addTab(
		Main.getText( "title.settings.screen" ),
		panelScreen );

    GridBagConstraints gbcScreen = new GridBagConstraints(
					0, 0,
					1, 1,
					0.0, 0.0,
					GridBagConstraints.WEST,
					GridBagConstraints.NONE,
					new Insets( 5, 5, 0, 5 ),
					0, 0 );

    panelScreen.add(
		new JLabel( Main.getText( "label.apect_ratio" ) ),
		gbcScreen );

    this.comboAspectRatio = new JComboBox<>(
			new String[] {
				BCScreen.VALUE_ASPECT_RATIO_4_3,
				BCScreen.VALUE_ASPECT_RATIO_16_10 } );
    this.comboAspectRatio.setEditable( false );
    updComboAspectRatio( Main.getProperty( BCScreen.PROP_ASPECT_RATIO ) );
    gbcScreen.gridx++;
    panelScreen.add( this.comboAspectRatio, gbcScreen );

    gbcScreen.gridx = 0;
    gbcScreen.gridy++;
    panelScreen.add(
		new JLabel( Main.getText( "label.color_at_start.fg" ) ),
		gbcScreen );

    this.comboColorAtStartFg = createColorComboBox();
    setSelectedIndex(
		this.comboColorAtStartFg,
		BCScreen.getFgColorNumSetting() );
    gbcScreen.gridx++;
    panelScreen.add( this.comboColorAtStartFg, gbcScreen );

    gbcScreen.gridx = 0;
    gbcScreen.gridy++;
    panelScreen.add(
		new JLabel( Main.getText( "label.color_at_start.bg" ) ),
		gbcScreen );

    this.comboColorAtStartBg = createColorComboBox();
    setSelectedIndex(
		this.comboColorAtStartBg,
		BCScreen.getBgColorNumSetting() );
    gbcScreen.gridx++;
    panelScreen.add( this.comboColorAtStartBg, gbcScreen );

    this.cbCursorAlwaysVisible = new JCheckBox(
		Main.getText( "option.cursor_always_visible" ),
		Main.getBooleanProperty(
			BCScreen.PROP_CURSOR_ALWAYS_VISIBLE,
			BCScreen.DEFAULT_CURSOR_ALWAYS_VISIBLE ) );
    gbcScreen.insets.top  = 10;
    gbcScreen.insets.left = 5;
    gbcScreen.gridwidth   = GridBagConstraints.REMAINDER;
    gbcScreen.gridx       = 0;
    gbcScreen.gridy++;
    panelScreen.add( this.cbCursorAlwaysVisible, gbcScreen );

    this.cbGoto950withoutCLS = new JCheckBox(
		Main.getText( "option.goto950_without_cls" ),
		Main.getBooleanProperty(
			BCRuntime.PROP_GOTO950_WITHOUT_CLS,
			BCRuntime.DEFAULT_GOTO950_WITHOUT_CLS ) );
    gbcScreen.insets.top = 0;
    gbcScreen.gridy++;
    panelScreen.add( this.cbGoto950withoutCLS, gbcScreen );


    // Tab warnings
    JPanel panelWarn = new JPanel();
    panelWarn.setLayout( new GridBagLayout() );
    this.tabbedPane.addTab(
		Main.getText( "title.settings.warnings" ),
		panelWarn );

    GridBagConstraints gbcWarn = new GridBagConstraints(
					0, 0,
					1, 1,
					0.0, 0.0,
					GridBagConstraints.WEST,
					GridBagConstraints.NONE,
					new Insets( 5, 5, 0, 5 ),
					0, 0 );

    panelWarn.add(
		new JLabel( Main.getText( "label.show_warning_when" ) ),
		gbcWarn );

    this.cbWarnNonAscii = new JCheckBox(
		Main.getText( "option.warning.non_ascii" ),
		ProgramParser.getWarnNonAsciiSetting() );
    gbcWarn.insets.top  = 0;
    gbcWarn.insets.left = 50;
    gbcWarn.gridy++;
    panelWarn.add( this.cbWarnNonAscii, gbcWarn );

    this.cbWarnSpecViolated = new JCheckBox(
		Main.getText( "option.warning.spec_violated" ),
		ProgramParser.getWarnSpecViolatedSetting() );
    gbcWarn.gridy++;
    panelWarn.add( this.cbWarnSpecViolated, gbcWarn );

    this.cbWarnRedeclaration = new JCheckBox(
		Main.getText( "option.warning.redeclaration" ),
		BCRuntime.getWarnRedeclarationSetting() );
    gbcWarn.gridy++;
    panelWarn.add( this.cbWarnRedeclaration, gbcWarn );

    this.cbWarnVarNotInitialized = new JCheckBox(
		Main.getText( "option.warning.var_not_initialized" ),
		BCRuntime.getWarnVarNotInitializedSetting() );
    gbcWarn.insets.bottom = 5;
    gbcWarn.gridy++;
    panelWarn.add( this.cbWarnVarNotInitialized, gbcWarn );


    // Tab end of record bytes
    JPanel panelEOR = new JPanel();
    panelEOR.setLayout( new GridBagLayout() );
    this.tabbedPane.addTab(
		Main.getText( "title.settings.eor_bytes" ),
		panelEOR );

    GridBagConstraints gbcEOR = new GridBagConstraints(
					0, 0,
					1, 1,
					0.0, 0.0,
					GridBagConstraints.WEST,
					GridBagConstraints.NONE,
					new Insets( 5, 5, 0, 5 ),
					0, 0 );

    panelEOR.add(
		new JLabel( Main.getText( "label.eor_bytes" ) ),
		gbcEOR );

    ButtonGroup grpEOR = new ButtonGroup();

    this.rbEorCR = new JRadioButton( Main.getText( "option.eor.cr" ) );
    grpEOR.add( this.rbEorCR );
    gbcEOR.insets.top  = 0;
    gbcEOR.insets.left = 50;
    gbcEOR.gridy++;
    panelEOR.add( this.rbEorCR, gbcEOR );

    this.rbEorCRNL = new JRadioButton( Main.getText( "option.eor.crnl" ) );
    grpEOR.add( this.rbEorCRNL );
    gbcEOR.gridy++;
    panelEOR.add( this.rbEorCRNL, gbcEOR );

    this.rbEorNL = new JRadioButton( Main.getText( "option.eor.nl" ) );
    grpEOR.add( this.rbEorNL );
    gbcEOR.gridy++;
    panelEOR.add( this.rbEorNL, gbcEOR );

    setEOR( BCRuntime.getEndOfRecord() );


    // Buttons
    JPanel panelBtn   = new JPanel( new GridLayout( 1, 3, 5, 5 ) );
    gbc.insets.left   = 10;
    gbc.insets.right  = 10;
    gbc.insets.bottom = 10;
    gbc.weightx       = 0.0;
    gbc.weighty       = 0.0;
    gbc.fill          = GridBagConstraints.NONE;
    gbc.gridy++;
    add( panelBtn, gbc );

    this.btnOK = new JButton( Main.getText( "action.ok" ) );
    panelBtn.add( this.btnOK );

    this.btnReset = new JButton( Main.getText( "action.reset" ) );
    panelBtn.add( this.btnReset );

    this.btnCancel = new JButton( Main.getText( "action.cancel" ) );
    panelBtn.add( this.btnCancel );


    // Window size
    pack();
    UIUtil.setParentCentered( this );
    setResizable( false );
  }


	/* --- Private Methods --- */

  // the colors has to be in BASICODE order!
  private static JComboBox<String> createColorComboBox()
  {
    JComboBox<String> combo = new JComboBox<>();
    combo.setEditable( false );
    combo.addItem( Main.getText( "color.black" ) );
    combo.addItem( Main.getText( "color.blue" ) );
    combo.addItem( Main.getText( "color.red" ) );
    combo.addItem( Main.getText( "color.magenta" ) );
    combo.addItem( Main.getText( "color.green" ) );
    combo.addItem( Main.getText( "color.cyan" ) );
    combo.addItem( Main.getText( "color.yellow" ) );
    combo.addItem( Main.getText( "color.white" ) );
    return combo;
  }


  private void doApply()
  {
    String aspectRatio = null;
    Object obj         = this.comboAspectRatio.getSelectedItem();
    if( obj != null ) {
      aspectRatio = obj.toString();
    }
    if( aspectRatio == null ) {
      aspectRatio = BCScreen.VALUE_ASPECT_RATIO_4_3;
    }
    Main.setProperty(
		BCScreen.PROP_ASPECT_RATIO,
		aspectRatio );
    Main.setProperty(
		BCScreen.PROP_COLOR_AT_START_BG,
		this.comboColorAtStartBg.getSelectedIndex() );
    Main.setProperty(
		BCScreen.PROP_COLOR_AT_START_FG,
		this.comboColorAtStartFg.getSelectedIndex() );
    Main.setProperty(
		BCScreen.PROP_CURSOR_ALWAYS_VISIBLE,
		this.cbCursorAlwaysVisible.isSelected() );
    Main.setProperty(
		BCScreen.PROP_INPUT_REVERSE_CASE,
		this.cbBCInputReverseCase.isSelected() );
    Main.setProperty(
		BCRuntime.PROP_GOTO950_WITHOUT_CLS,
		this.cbGoto950withoutCLS.isSelected() );
    Main.setProperty(
		BCRuntime.PROP_WAIT_DURATION_WHEN_SOUND_OFF,
		this.cbWaitDurationWhenSoundOff.isSelected() );
    Main.setProperty(
		BCRuntime.PROP_WARN_REDECLARATION,
		this.cbWarnRedeclaration.isSelected() );
    Main.setProperty(
		BCRuntime.PROP_WARN_VAR_NOT_INITIALIZED,
		this.cbWarnVarNotInitialized.isSelected() );
    Main.setProperty(
		ProgramParser.PROP_IGNORE_CASE,
		this.cbBCParserIgnoreCase.isSelected() );
    Main.setProperty(
		ProgramParser.PROP_WARN_NON_ASCII,
		this.cbWarnNonAscii.isSelected() );
    Main.setProperty(
		ProgramParser.PROP_WARN_SPEC_VIOLATED,
		this.cbWarnSpecViolated.isSelected() );
    String eorValue = "";
    if( this.rbEorCR.isSelected() ) {
      eorValue = BCRuntime.VALUE_EOR_CR;
    } else if( this.rbEorCRNL.isSelected() ) {
      eorValue = BCRuntime.VALUE_EOR_CRNL;
    } else if( this.rbEorNL.isSelected() ) {
      eorValue = BCRuntime.VALUE_EOR_NL;
    }
    Main.setProperty( BCRuntime.PROP_DATA_FILE_EOR, eorValue );
    this.mainFrm.settingsChanged();
    doClose();
  }


  private void doClose()
  {
    setVisible( false );
    dispose();
  }


  private void doReset()
  {
    updComboAspectRatio( BCScreen.DEFAULT_ASPECT_RATIO );
    setSelectedIndex(
		this.comboColorAtStartBg,
		BCScreen.DEFAULT_COLOR_AT_START_BG );
    setSelectedIndex(
		this.comboColorAtStartFg,
		BCScreen.DEFAULT_COLOR_AT_START_FG );
    this.cbBCInputReverseCase.setSelected(
		BCScreen.DEFAULT_INPUT_REVERSE_CASE );
    this.cbBCParserIgnoreCase.setSelected(
		ProgramParser.DEFAULT_IGNORE_CASE );
    this.cbCursorAlwaysVisible.setSelected(
		BCScreen.DEFAULT_CURSOR_ALWAYS_VISIBLE );
    this.cbGoto950withoutCLS.setSelected(
		BCRuntime.DEFAULT_GOTO950_WITHOUT_CLS );
    this.cbWaitDurationWhenSoundOff.setSelected(
		BCRuntime.DEFAULT_WAIT_DURATION_WHEN_SOUND_OFF );
    this.cbWarnNonAscii.setSelected(
		ProgramParser.DEFAULT_WARN_NON_ASCII );
    this.cbWarnSpecViolated.setSelected(
		ProgramParser.DEFAULT_WARN_SPEC_VIOLATED );
    this.cbWarnRedeclaration.setSelected(
		BCRuntime.DEFAULT_WARN_REDECLARATION );
    this.cbWarnVarNotInitialized.setSelected(
		BCRuntime.DEFAULT_WARN_VAR_NOT_INITIALIZED );
    setEOR( BCRuntime.getDefaultEndOfRecord() );
  }


  private void setEOR( BCRuntime.EOR eor )
  {
    switch( eor ) {
      case CR:
	this.rbEorCR.setSelected( true );
	break;
      case CRNL:
	this.rbEorCRNL.setSelected( true );
	break;
      case NL:
	this.rbEorNL.setSelected( true );
	break;
    }
  }


  private static void setSelectedIndex( JComboBox<?> combo, int idx )
  {
    try {
      combo.setSelectedIndex( idx );
    }
    catch( IllegalArgumentException ex ) {}
  }


  private void updComboAspectRatio( String aspectRatio )
  {
    int idx = 0;
    if( aspectRatio != null ) {
      if( aspectRatio.equals( BCScreen.VALUE_ASPECT_RATIO_16_10 ) ) {
	idx = 1;
      }
    }
    setSelectedIndex( this.comboAspectRatio, idx );
  }
}
