/*
 * (c) 2025 Jens Mueller
 *
 * JBASICODE
 *
 * Debugger Component
 */

package jbasicode.ui;

import java.util.List;
import javax.swing.table.AbstractTableModel;
import jbasicode.Main;


public class VarTableModel extends AbstractTableModel
{
  private String        colName0;
  private String        colName1;
  private List<VarView> rows;


  public VarTableModel()
  {
    this.colName0 = Main.getText( "title.table.var_decl" );;
    this.colName1 = Main.getText( "title.table.var_value" );
    this.rows     = null;
  }


  public void setRows( List<VarView> rows )
  {
    this.rows = rows;
    fireTableDataChanged();
  }


  public void clear( boolean updView )
  {
    this.rows = null;
    fireTableDataChanged();
  }


	/* --- TableModel --- */

  @Override
  public Class<?> getColumnClass( int col )
  {
    return (col == 0) || (col == 1) ? String.class : Object.class;
  }


  @Override
  public int getColumnCount()
  {
    return 2;
  }


  @Override
  public String getColumnName( int col )
  {
    String rv = "";
    switch( col ) {
      case 0:
	rv = this.colName0;
	break;
      case 1:
	rv = this.colName1;
	break;
    }
    return rv;
  }


  @Override
  public int getRowCount()
  {
    List<VarView> rows = this.rows;
    return rows != null ? rows.size() : 0;
  }


  @Override
  public Object getValueAt( int row, int col )
  {
    Object        rv   = null;
    List<VarView> rows = this.rows;
    if( rows != null ) {
      if( (row >= 0) && (row < rows.size())
	  && (col >= 0) && (col < 2) )
      {
	VarView varView = rows.get( row );
	if( varView != null ) {
	  switch( col ) {
	    case 0:
	      rv = varView.getDeclarationText();
	      break;
	    case 1:
	      rv = varView.getValueView();
	      break;
	  }
	}
      }
    }
    return rv;
  }


  @Override
  public boolean isCellEditable( int row, int col )
  {
    return false;
  }
}
