/*
 * (c) 2025 Jens Mueller
 *
 * JBASICODE
 *
 * View of a variable
 */

package jbasicode.ui;


public class VarView implements Comparable<VarView>
{
  private String declText;
  private String valueView;


  public VarView( String declText, Object valueView )
  {
    this.declText  = declText;
    this.valueView = toValueString( valueView );
  }


  public String getDeclarationText()
  {
    return this.declText;
  }


  public String getValueView()
  {
    return this.valueView;
  }


  public static String toValueString( Object value )
  {
    String text = null;
    if( value != null ) {
      text = value.toString();
      if( (text != null) && !(value instanceof Number) ) {
	int len = text.length();
	if( len > 0 ) {
	  StringBuilder buf = new StringBuilder( 128 );
	  for( int i = 0; i < len; i++ ) {
	    if( len > 100 ) {
	      buf.append( "..." );
	      break;
	    }
	    char ch = text.charAt( i );
	    switch( ch ) {
	      case '\u0000':
		buf.append( "\\0" );
		break;
	      case '\n':
		buf.append( "\\n" );
		break;
	      case '\r':
		buf.append( "\\r" );
		break;
	      case '\t':
		buf.append( "\\t" );
		break;
	      default:
		if( Character.isDefined( ch ) ) {
		  buf.append( ch );
		} else {
		  buf.append( String.format( "\\u%04X", (int) ch ) );
		}
	    }
	  }
	  text = buf.toString();
	}
      }
    }
    return text != null ? text : "";
  }


	/* --- Comparable --- */

  @Override
  public int compareTo( VarView varView )
  {
    return this.declText.compareTo( varView.declText );
  }
}
